local _G = getfenv(0)

local sct,db,init

local crit = CreateFrame("Frame")

local major,minor = tonumber(GetAddOnMetadata("Vengance", "Version")),tonumber(string.match("$Revision: 60312 $", "[0-9]+"))

local name,realm = UnitName("player"),GetRealmName()

local database = {
	[realm] = {
		[name] = {
			["Version"] = {major,minor},
			["Output"] = true,
			["CritDB"] = {
			},
			["HitDB"] = {
			},
		},
	},
}


local printf = function(str, ...)
	str = string.format(str, ...)
	ChatFrame1:AddMessage("|cffFF0099Vengance|r : "..str)
end

local print = function(...)
	local text = ""
	if select("#", ...) > 1 then
		for i = 1,select("#", ...) do
			text = text.." "..tostring(select(i, ...))
		end
	else
		text = select(1, ...)
	end
	ChatFrame1:AddMessage("|cffFF0099Vengance|r: "..text)
end

local versionctl = function()
	if db.Version[1] ~= major then
		printf("Version missmatch in db, %d expected %d", db.Version[1],major)
		db.Version[1] = major
		db.Version[2] = minor
		return true
	end
end

local SCTcheck = function(self, ...)
	if select(4, GetAddOnInfo("MikScrollingBattleText")) then
		return "MSBT"
	elseif select(4, GetAddOnInfo("Parrot")) then
		return "PARROT"
	elseif select(4, GetAddOnInfo("sct")) then
		return "SCT"
	else
		return
	end
end

local initdb = function(self, force)
	init = _G.VenganceDB
	if not init then
		print("Importing db from defaults")
		init = database
		_G.VenganceDB = init
	end
	--new char in variables?
	if not init[realm] or force then
		print("New Character detected, importing fresh db")
		init[realm] = database[realm]
		if not init[realm][name] or force then
			init[realm][name] = database[realm][name]
			_G.VenganceDB = init
		end
	end
	db = init[realm][name]
end
local sort = function(self, ...)
	local t,keys = {},{}
	for spell,dmg in pairs(db.CritDB) do
		keys[# keys + 1] = dmg
		t[dmg] = spell
	end
	table.sort(keys, function(a,b) return db.CritDB[b] > db.CritDB[a] end)
	for key,dmg in ipairs(keys) do
		print(dmg)
	end
end

crit.ADDON_LOADED = function(self, event, arg1)
	if arg1 == "Vengance" then
		initdb()
		--print("initdb")
	end
end

local output = function(self, spell, dmg)
	if db["Output"] then
		local str = string.format("New Record: %s - %d", spell, dmg)
		if sct == "SCT" then
			local col = { ["r"] = 255, ["g"] = 0, ["b"] = 153 }
			SCT:DisplayCustomEvent(str, col, sticky, SCT.FRAME1)
			return
		elseif sct == "MSBT" then
			MikSBT.DisplayMessage(str, nil, true, 255, 0, 153)
			return
		elseif sct == "PARROT" then
			Parrot:ShowMessage(str)
			return
		else
			printf("New record: %s - %d", spell,dmg)
		end
	end
end

crit.PLAYER_LOGIN = function(self, event, arg1)
	sct = SCTcheck()
	versionctl()
end

crit.parser = function(self, event, ...)
end
crit.CHAT_MSG_COMBAT_SELF_HITS = function(self, event, arg1)
	if arg1:match("crits") then
		local spell,mob,dmg = arg1:match("Your (.+) crits (.+) for (%d+)")
		dmg = tonumber(dmg)
		if not db.CritDB[spell] or (dmg > db.CritDB[spell]) then
			db.CritDB[spell] = dmg
			output(self, spell, dmg)
			return
		end
	elseif arg1:match("You crit") then
		local dmg,spell = arg1:match("(%d+)"),"Attack"
		dmg = tonumber(dmg)
		if not db.CritDB[spell] or (dmg > db.CritDB[spell]) then
			db.CritDB[spell] = dmg
			output(self, spell, dmg)
			return
		end
	elseif arg1:match("hits") then
		local dmg,spell = arg1:match("(%d+)"),"Attack"
		dmg = tonumber(dmg)
		if not db.HitDB[spell] or (dmg > db.HitDB[spell]) then
			db.HitDB[spell] = dmg
		end
	end
end


crit.CHAT_MSG_SPELL_SELF_DAMAGE = function(self, event, arg1)
	if arg1:match("crits") then
		local spell,mob,dmg = arg1:match("Your (.+) crits (.+) for (%d+)")
		dmg = tonumber(dmg)
		if not db.CritDB[spell] or (dmg > db.CritDB[spell]) then
			db.CritDB[spell] = dmg
			output(self, spell, dmg)
		end
	elseif arg1:match("hits") then
		local spell,mob,dmg = arg1:match("Your (.+) hits (.+) for (%d+)")
		dmg = tonumber(dmg)
		if not db.HitDB[spell] or (dmg > db.HitDB[spell]) then
			db.HitDB[spell] = dmg
		end
	end
end

crit.CHAT_MSG_SPELL_PERIODIC_CREATURE_DAMAGE = function(self, event, arg1)
	if arg1:match("suffers") then
		local dmg,spell = arg1:match("(%d+).*from your (.+)")
		dmg = tonumber(dmg)
		spell = spell.." (dot)"
		if not db.HitDB[spell] or (dmg > db.HitDB[spell]) then
			db.HitDB[spell] = dmg
			output(self, spell, dmg)
		end
	end
end

crit:SetScript("OnEvent", function(self, event, ...)
	self[event](self, event, ...)
end)

crit:RegisterEvent("PLAYER_LOGIN")
crit:RegisterEvent("ADDON_LOADED")
crit:RegisterEvent("CHAT_MSG_COMBAT_SELF_HITS")
crit:RegisterEvent("CHAT_MSG_SPELL_SELF_DAMAGE")
crit:RegisterEvent("CHAT_MSG_SPELL_PERIODIC_CREATURE_DAMAGE")
--[[Steal your carbon, this is based on]]
SLASH_VENGANCE1 = "/vengance"
SLASH_VENGANCE2 = "/veng"
SlashCmdList.VENGANCE = function(input)
	local cmd,rest = input:match("s*(%S+)%s*(.*)$")
	local handle = cmd and crit["CMD_"..cmd:upper()]
	if handle then
		handle(crit, rest)
	else
		print("Vengance - Minimal Crit log tracking")
		print("Commands:")
		print("         List: Lists the current crit records")
		print("         Output: Enable or disable output")
		print("         Test: Fires a test output")
		print("         Reset: Resets the db")
	end
end


crit.CMD_LIST = function(self, cmd)
	if cmd == "" then
		for k,v in pairs(db.HitDB) do
			if db.CritDB[k] then
				printf("%s|cffFF0099 // |r%d|cffFF0099 // |r%d", k,v,db.HitDB[k])
			else
				printf("%s |cffFF0099//|r %d", k,v)
			end
		end
	else
		if db.CritDB[cmd] then
			printf("%s - %d", cmd, db.CritDB[cmd])
		else
			print("Doesnt exist in DB")
		end
	end
end

crit.CMD_TEST = function(self, cmd)
	output(self, "THIS IS A TEST STRING LULZ", 123)
end

crit.CMD_RESET = function(self, cmd)
	if cmd == "true" then
		for k,v in pairs(db.CritDB) do
			db.CritDB[k] = nil
		end
		print("Crit db reset")
	elseif cmd == "hard" then
		VenganceDB = nil
		initdb()
	else
		print("use reset true to reset crits, or reset hard to completly reset")
	end
end

crit.CMD_OUTPUT = function(self, cmd)
	db.Output = not db.Output
	printf("Output %s", (db.Output and "enabled") or "disabled")
end

crit.CMD_SORT = function(self, cmd)
	sort()
end
